#import warnings
from Aboutn import tela

representacao= (5,) + ((4,) * (len(tela.sentidos) + 1))
algarismos	 = "01"#'ai'
f_max	= 100
tamanho	=   0 #73 == 5 + (4 * (len(tela.sentidos) + 1))
for r in representacao:
	tamanho += r

def verificar (*ind):
	e = 0
	for i in ind:
		try:
			if i.erro == None or i.verificar():
				e += 1
		except AttributeError:
			if i == None or len(i) != tamanho:
				e += 1
	return e

def gerar (tipo=str):
	if tipo==str:
		c = ''
		d = tamanho
		while d > 0:
			d -= 1
			c += algarismos[int(len(algarismos)*tela.random())]
		return c
	return tipo(gerar())

def mutar (c):
	return c

def cruzar (a,b):
	try:
		c = 1 + int(tela.random() * (tamanho - 2))
		return a[:c] + b[c:]
	except TypeError:
		if type(a) == str:
			return b + a
		#	a, b = b, a
		return a + b

def reverter (c):
	try:
		return c.reverse()
	except AttributeError:
		return c[len(c)::-1]

def negativar (c):
	try:
		return -c#.__neg__()
	except TypeError:
		a = ''
		for b in c.__str__():
			try:
				a += algarismos[len(algarismos) - (algarismos.find(b) + 1)]
			except IndexError:
				a += b			
		return a

def decimal (b):
#	if type (b) != str:
#		b = str(b)
	a = c = 0
	while c < len(b):
		if not b[c] in algarismos:
			print('Dígito %s sem valor em %s' %(b[c],b))
			#warnings.warn('Dígito %s sem valor em %s' %(b[c],b),SyntaxWarning)
		a = (a*len(algarismos)) + algarismos.find(b[c])
		c += 1
	return a

class Quadrado:

	y = x = 0
	f = s = c = cor = erro = correr = None

	def __ne__ (self,outro):
		return not self.__eq__(outro)
	def __eq__ (self,outro):
		try:
			if None != self.c and outro.c != None:
				return self.c == outro.c
	
			return self.y == outro.y and self.x == outro.x #and self.s == outro.s
		except AttributeError:
			#print('Não sou None!')
			return False
		
	def __lt__ (self,outro):
		try:
			if type(outro) != Quadrado:
				return None
			return self.f < outro.f
		except TypeError:
			return self.y < outro.y or (self.y == outro.y and self.x < outro.x)
		#	return self.c < outro.c
	def __gt__ (self,outro):
		try:
			if type(outro) != Quadrado:
				return None
			return self.f > outro.f
		except TypeError:
			return self.y > outro.y or (self.y == outro.y and self.x > outro.x)
		#	return self.c > outro.c
	def __ge__ (self,outro):
		return not self.__lt__(outro)
	def __le__ (self,outro):
		return not self.__gt__(outro)
	def __len__ (self):
		try:
			return self.c.__len__()
		except AttributeError:
			return -1
	def __iter__ (self):
		try:
			return self.c.__iter__()
		except AttributeError:
			return NotImplemented
	def __contains__ (self,gene):
		try:
			return self.c.__contains__(gene)
		except AttributeError:
			return None
	
	def __repr__ (self):
		return self.__str__()
	def __str__ (self):
		if None==self.c:
			res = "None, False,%d,%d"%(self.x,self.y)
		else:
			res = self.c.__repr__()#'"%s"' %self.c
			if self.f != None:
				res += ', x=%d,y=%d, apto=%f' %(self.x, self.y, self.f)
		res = 'Quadrado(%s)' %res
		if __name__ != '__main__':
			res = 'quadrado.%s' %res
		return res
	
	def __init__ (self, cromossomo=None, mestra=None,x=None,y=None,cor=None,apto=None):
		while type(cromossomo) == Quadrado:
#			if self.erro == None:
#				self.erro = cromossomo.erro
			if apto == None:
				try:
					apto = cromossomo.f
				except AttributeError:
					pass
			if None == x:
				x = cromossomo.x
			if None == y:
				y = cromossomo.y
			if None == cor:
				cor = cromossomo.cor
			if None == mestra:
				mestra = cromossomo.s
			cromossomo = cromossomo.c
		try:
			if type(mestra) != tela.Tela:
				mestra = mestra.s
		except AttributeError:
			mestra = None
		if None==mestra==cromossomo:
			mestra = tela.Tela(self)
			if None==x:
				x = tela.qx
			if None==y:
				y = tela.qy
		else:
#			if None==cromossomo:
#				cromossomo = gerar()
#			else:
#				self.__len__ = self.c.__len__
#				self.__contains__ = self.c.__contains__
			self.c = cromossomo
		self.mestra(mestra,cor)
		if self.analisar() and apto != None:
			self.f = apto
		if y != None:
			self.y = y
		if x != None:
			self.x = x

	def __abs__ (self):
		return decimal(self.c)#.__abs__()

	def __add__ (self,outro):
		if verificar(self,outro):#self.verificar() or Quadrado(outro).verificar():
			print('Cruzamento inválido')
			return None

		try:
			c = cruzar(self.c,outro.c)
		except AttributeError:
			c = cruzar(self.c,outro)
		return Quadrado(c,self.s,cor=self.cor)
	#	return Quadrado(c,self.s,self.x,self.y,self.cor)

	def __neg__ (self):
		return Quadrado(negativar(self.c),self.s,self.x,self.y,self.cor)

	def reverse (self):
		return Quadrado(reverter(self.c),self.s,self.x,self.y,self.cor)

	def index (self,gene):
		return self.c.index(gene)
	def find (self, gene):
		try:
			return self.c.find(gene)
		except AttributeError:
			return -2
	def count (self,gene):
		try:
			return self.c.count(gene)
		except AttributeError:
			return  0

	def copy (self):
		return Quadrado(self.c,self.s,cor=self.cor)

	def mestra (self,mestra=None,cor=None):
		if None != cor:
			self.cor = cor
		if None == mestra:
			return self.s
		self.s = mestra

	def verificar (self):
		return self.erro or self.c == None or len(self.c) != tamanho

	def analisar (self):
#		while type(self.c) == Quadrado:
#			self.c = self.c.c
		if self.verificar():
			#print(self,':',self.c,'inválido como Cromossomo')
			return None#NotImplemented
		r = 0
		rotas = []
		for l in representacao:
			#print(self.c[r:r+l],decimal(self.c[r:r+l]))
			rotas.append(decimal(self.c[r:r+l]))
			r += l
		self.x = rotas.pop(0)
		self.y = rotas.pop(0)
		self.rotas = {}
		r = len(tela.sentidos)#rotas
		try:
			while r > 0:
				r += -1
				self.rotas[tela.sentidos[r]] = tela.sentidos[rotas[r]]
		except Exception:
			self.erro = True
		else:
			self.erro = False
		finally:
			return not self.erro

	def avaliar (self):
		try:
		#	d = ((self.y-self.s.q.y)**2 + (self.x-self.s.q.x)**2)**(1/2)
			self.f = f_max*(1-((((self.y-self.s.q.y)**2 + (self.x-self.s.q.x)**2)**(1/2))/tela.d_max))
		#	self.f = 100*(1-((((self.y-self.s.q.y)**2 + (self.x-self.s.q.x)**2)**(1/2))/tela.d_max))
		#	self.f = (101**(1-(d/d_max))) - 1
		#	self.f = (101/(101**(d/d_max)))-1
		except AttributeError:
			self.f = 0
			print(str(self) + '.avaliar(): Mestra, seu Protagonista ou as Coordenadas do último não encontrado(a)(s)')
			#warnings.warn(str(self) + '.avaliar(): Mestra, seu Protagonista ou as Coordenadas do último não encontrado(a)(s)',RuntimeWarning)
			#RuntimeWarning('Mestra, seu Protagonista ou as Coordenadas do último não encontrado(a)(s)')
		return self.f
	
	def localizar (self):
		try:
			coord = [self.x - self.s.q.x, self.y - self.s.q.y]
			z = len(coord)
			while z > 0:
				z += -1
				if coord[z] < -2:
					coord[z] = -2
				if coord[z] > 2:
					coord[z] = 2
			return tuple(coord)
		except AttributeError:
			print(str(self) + '.localizar(): Mestra, seu Protagonista ou as Coordenadas do último não encontrado(a)(s)')

	def mover (self,dex=0,dey=0):
		i = self.s.colorir(dex,dey)
		j = self.s.colorir(self.x,self.y,self.cor)
		return i and j

	def ir (self,dx=None,dy=None,para=None,permissao=False):
		if not (self.correr or permissao):
			return None

		if para == None:
			try:
				if None==dx==dy:
					dx,dy = self.rotas[self.localizar()]
			except KeyError:
				dy=dx = 0
			#	dx,dy = tela.sentidos[int(len(tela.sentidos)*tela.random())]
				#print('No ponto cego de',str(self))

			self.x,dx = self.x+dx,self.x
			self.y,dy = self.y+dy,self.y
		else:
			try:
				dx,dy = self.x,self.y
				self.x,self.y = para[:2]
			except TypeError:
				pass
			except ValueError:
				pass
			
		if self.y >= tela.my:
			self.y = tela.my - 1
		if self.x >= tela.mx:
			self.x = tela.mx - 1
		if self.x < 0:
			self.x = 0
		if self.y < 0:
			self.y = 0

		if not (self.mover(dx,dy) or permissao):
			#print('Movimentação de',self,'não concluída.')
			return False
		return True

#	def travar (self,v=True):
#		self.vencido = v

	def andar (self):
		while self.s.n.correr and self.correr and self.ir():
			if self.s.q == self:
				self.s.n.continuar(self)
			tela.sleep(0.55 + tela.random())
		#print(self,'parou de andar.')

	def iniciar (self):
		self.correr = True
		if self.c != None:
			tela.Thread(None,self.andar,str(self)).start()

	def pausar (self):
		self.correr = False
	



VERMELHO='RED'; 	VER = {"bg":VERMELHO,"fg":VERMELHO}
BRANCO = 'WHITE'; 	BRA = {"bg": BRANCO, "fg":BRANCO}
PRETO  = 'BLACK'; 	PRE = {"bg": PRETO, "fg": PRETO}

#	diferença entre x,y da coisa relativa e do objeto de origem (ex: player relativo ao bot)
	#		(1,0),(1,1),(0,1),(-1,1),(-1,0),(-1,-1),(0,-1),(1,-1),(2,0),(2,2),(0,2),(-2,2),(-2,0),(-2,-2),(0,-2),(2,-2), (2, -1), (2, 1), (-1, 2), (1, 2), (-2, -1), (-2, 1), (-1, -2), (1, -2)
sentidos = [(1,0),(1,1),(0,1),(-1,1)]
for i in (-1,2):
	j,k = 0,len(sentidos)
	while j < k:
		sentidos.append((sentidos[j][0]*i,sentidos[j][1]*i))
		#print(sentidos[len(sentidos)-1])
		j += 1
#k = 1
#for i in range(len(sentidos)//2,len(sentidos)):
#	if 0 in sentidos[i]:
#		for j in range(2):#"''":
#			j = list(sentidos[i])
#			k = -k
		#	k *= -1
#			j[j.index(0)] = k
#			sentidos.append(tuple(j))
i,j = 1,2
z = '"'*2#range(2)
for x in z:
	for y in z:
		sentidos.append((i,j))
		sentidos.append((j,i))
		i = -i
	j = -j

sentidos= tuple(sentidos)		
mx,my	=	32,16
qx,qy	=	mx//2, my//2
d_max	= ((mx-1)**2 + (my-1)**2)**(1/2)
#teto = mx,my
#quad 	= [qx,qy]
#vence 	= [0,0]
#qtd 	= 2

print('Tupla de coordenadas dos %d sentidos:'%len(sentidos),sentidos,'\nCoordenadas máximas:\t %d,%d' %(mx,my),'\nDistância máxima:\t', d_max)

from tkinter import BOTH, RIGHT, LEFT, Tk, Frame, Button, Label
from threading import Thread
from time import sleep, time
from random import random
from Aboutn.quadro import Quadro#, quadrado

'''
 #	aptidão
def avaliar (d,g=None): 
	global quad
	if g == None:
		global d_max
		global f_max
		return f_max*(1-(d/d_max))
	return avaliar(((d-quad[0])**2 + (g-quad[1])**2)**(1/2))

c = d_max
while c >= 0:
	print(c,avaliar(c))
	c -= 1
c = 0
while c <= d_max:
	print(c,avaliar(c))
	c += 1

def gerar ():
	return [int(mx*random()),int(my*random())]


def andar (*coord):
	quadrados[quad[1]][quad[0]].config(**BRA)
	c = 0
	try:
		while len(coord)>c:
			if teto[c] > coord[c] and coord[c] >= 0:
				quad[c] = coord[c]
			c += 1
	except Exception:
		pass
	quadrados[quad[1]][quad[0]].config(**VER)
'''

def xadrez (mestre, c=8,h=None,espera=0,matriz=[],ligue=False,*ligargs):
	try:
		mestre = Frame(mestre)
		mestre.pack(fill=BOTH)
		t = time()
		if (h==None):
			h = c
		y = h
		while y>0:
			x = c
			m = Frame(mestre)
			m.pack(fill=BOTH)
			matriz.insert(0,[])
			while x>0:
			#	l = Frame(m,width=mestre.winfo_screenwidth()//n,height=mestre.winfo_screenheight()//n)
			#	l.pack()
				matriz[0].insert(0,Button(m,text=(x,y)))
				matriz[0][0].pack(fill=BOTH, side=RIGHT)
				matriz[0][0].coord = x-1,y-1
			#	matriz[0][0].ir = lambda col=x-1,lin=y-1: funcao(col,lin)#matriz[lin][col].config(**VER)#print(pos)
				#print(x,y)
				sleep(espera/(x*y))
				x -= 1
			y -= 1
	except Exception:
		return print('Construção abortada')

	if ligue:
		t = time() - t
		mestre.master.title("About n []")
		print ('Espera máxima:\t%f\nTempo total:\t%f\nTempo médio:\t%f\nTotal:\t%d botões\n'%(espera,t,t/(h*c),c*h))
		ativar(matriz,*ligargs)
	return matriz

def iniciar (mestre, x=16,y=None,*imolados,tempo=1/13,botoes=[],inicio=lambda: print('\t\aFunção para ativação não fornecida'),comando=lambda para: print('\tMétodo para pressão não fornecido',*para),fechar=input):
	for i in imolados:
		try:
			i.destroy()
		except Exception:
			print('\t\aNão foi possível destruir o elemento',i)
	Thread(None, xadrez, "Breve-montagem-ambiental", (mestre,x,y,tempo,botoes,True,comando,inicio)).start()
	print('Tupla de objetos destruídos:',imolados)
	mestre.protocol("WM_DELETE_WINDOW", fechar)
	mestre.title('About n')
	

def ativar (tabuleiro,pressionar=None,principiar=input):
	try:
		for l in tabuleiro:
			for c in l:
				if pressionar:
					c.config(command=lambda to=c.coord: pressionar(para=to))#c.ir)#lambda pos=c.coord: print(pos))
				c.config(**BRA)
			c.master.config(bg=BRANCO)
		c.master.master.config(bg=BRANCO)
		c.master.master.master.config(bg=BRANCO)
		c.master.master.master.title ('About n [Squares]')
	#	c.master.master.master.protocol("WM_DELETE_WINDOW", DISABLED)
	except Exception:
		print('\t\aUm erro ocorreu na construção da tela branca, no caso de permanência do cinza claro padrão em algum elemento, por favor, repita esta operação.')
	principiar()
"""	cuidar()

def cuidar (b=None,bots=[]):
	global vence
	global quad
	global qtd
	global BRA
	global PRE
	global PRETO
	global BRANCO

	if b != None:
		while len(bots) > 0 and not quad in bots:
			quadrados[b[1]][b[0]].config(**BRA)
			dv = sentidos[int(random()*len(sentidos))]
			c = len(b)
			while c > 0:
				c	+= -1
				b[c]+= dv[c]
				if b[c] < 0:
					b[c] = 0
				if b[c] >= teto[c]:
					b[c] = teto[c] - 1
			quadrados[b[1]][b[0]].config(**PRE)
			print(b,avaliar(*b))
			if b != quad:
				sleep((11/12)-(random()/8))
			if b == quad:
				print('\a\t',b)
				bots.clear()
				qtd += 1#= len(bots) + 1
				vence,quad = b,[mx//2,my//2]
				sleep((12/11)+(random()/4))
				ativar(quadrados)
				return
		quadrados[b[1]][b[0]].config(**BRA)
		BRANCO,BRA,PRE,PRETO = PRETO,PRE,BRA,BRANCO
		return
	
	b = vence
	c = qtd
	while c>0:	
		if b != quad:
			bots.append(b)
		b = gerar()
		c -= 1
	if len(bots) < 2:
		b = bots
		cuidar(bots=b)
		print("Ação de emergência utilizada")
	else:
		print(len(bots),bots)
	for b in bots:
		Thread(None, lambda: cuidar(b,bots)).start()
	
a = Tk()
#a.config(bg='WHITE')
a.title("About 2") #a História de n Quadrados
#a.attributes('-fullscreen', True)
#a.geometry('%dx%d' %(a.winfo_screenwidth()//2, a.winfo_screenheight()//2))
#b = {'width':a.winfo_screenwidth()//9,'height':a.winfo_screenheight()//8,'master':a}
#b = Frame(**b),Frame(**b)
b = [Label(a,text="\n\n")]
b.extend((Button(a,command=lambda destruir=b: iniciar(a,mx,my,*destruir,botoes=quadrados),**VER,text=VERMELHO), Button(a,command=a.quit,bg=PRETO,text="n")))
b[1].pack(side=RIGHT)
b[2].pack()
b[0].pack()
b = '0123456789ABCDEF'
c = 0
while c < len(sentidos):
	for d in (b[c]*2).title():
		a.bind(d,lambda event,x=sentidos[c][0],y=sentidos[c][1]: andar(quad[0]+x,quad[1]+y))
	c += 1
#Thread(None, xadrez, None, (a,)).start()
#Thread(None, ).start()
a.mainloop()
"""

class Tela:

	q = n = b = tela = None
	def __gt__ (self, o):
		try:
			return (self.q > o.q or (self.q == o.q and self.n > o.n))
		except Exception:
			return NotImplemented
	def __lt__ (self, o):
		try:
			return (self.q < o.q or (self.q == o.q and self.n < o.n))
		except Exception:
			return NotImplemented
	def __le__ (self, o):
		return type(o) == Tela and not self.__gt__(o)
	def __ge__ (self, o):
		return type(o) == Tela and not self.__lt__(o)
	def __ne__ (self, o):
		return not (self.__ge__(o) and self.__le__(o))
	def __eq__ (self, o):
		return not self.__ne__(o)
	def __repr__ (self):
		return self.__str__()
	def __str__ (self):
		res = 'Tela(%s,%s)' %(self.q.__repr__(),self.n.__repr__())
		if __name__ != '__main__':
			res = 'tela.%s' %res
		return res

	def __init__ (self, vermelho=None, quadrados=None):
		if type(vermelho) == Tela:
			if quadrados == None:
				quadrados =	vermelho.n
			vermelho	=	vermelho.q#protagonista()
		if type(quadrados) == Quadro:
			quadrados.mestra(self)
		elif type(quadrados) in (tuple,list,set):
			quadrados = Quadro(self,quadrados)
		else:
			quadrados = Quadro(self)
		self.n = quadrados
		self.protagonista(vermelho)

	def protagonista (self,principal=None):
		if principal == None:
			return self.q
		self.q = principal
		if self != principal.s:
			self.q.mestra(self,VER)
	#	self.q.ir(para=(mx//2,my//2))

	def colorir (self,x,y,cor=BRA):
		try:
			self.b[y][x].config(**cor)
			return True
		except IndexError:
			#print('Posição [',x,y,'] inválida')
			return False
		except:
			#print('Erro em configurar o botão [',x,y,'] de', self.b)
			return 
	
	def iniciar (self,tela=None,quadrados=[]):
		if self.tela == None:
			if tela == None:
				tela = Tk()
			self.tela = tela
		self.tela.title('About 2')
		self.tela.protocol("WM_DELETE_WINDOW", self.tela.quit)
		self.b = quadrados
		b = [Label(tela,text='\n\n',bg=self.tela['bg'])]
		#print(b)
		b.append(Button(tela,command=lambda destruir=b: iniciar(self.tela,mx,my,*destruir,botoes=self.b,comando=self.q.ir,inicio=self.n.iniciar,fechar=self.reiniciar),**VER,text=VERMELHO))
		b.append(Button(tela,command=self.n.reiniciar,bg=PRETO,text='n'))
		b[1].pack(side=RIGHT)
		b[2].pack()
		b[0].pack()
		b = '0123456789ABCDEF'
		c = len(b)#sentidos) - 1
		while c > 0:
			c += -1
			for d in (b[c]*2).title():
				tela.bind(d,lambda event,to=sentidos[c]: self.q.ir(*to))
	#	for d in 'Zz':
	#		tela.bind(d,self.n.reiniciar)
	#	self.tela.bind('<space>',self.continuar)
		self.tela.bind('<Escape>',self.reiniciar)
		
		self.tela.mainloop()

	def reiniciar (self,event=0):
		try:
			self.n.pausar()
			self.b.pop(0).pop(0).master.master.destroy()
			self.b.clear()
			self.iniciar()
		except Exception:
			pass
			#print(self,'incapaz de reiniciar neste momento, tente novamente mais tarde.')

	def continuar (self,geracao=None):
		if len(self.n)%2==1:
			global BRANCO, PRETO
			BRANCO, PRETO = PRETO, BRANCO
			BRA['bg'],BRA['fg'],PRE['fg'],PRE['bg'] = PRE['bg'],PRE['fg'],BRA['fg'],BRA['bg']
		ativar(self.b,principiar=self.n.iniciar)


from Aboutn import quadrado
tamanho = 24#len(quadrado.tela.sentidos)#*2
#print('Aptidão máxima: \t',quadrado.f_max,'\nTamanho cromossômico:\t',quadrado.tamanho,'\t(%s)\nTamanho populacional:\t' %quadrado.algarismos,tamanho)

def popular (n=0,p=None):
	if p == None:
		p = []
	while len(p) < n//4:
		p.append(quadrado.gerar(quadrado.Quadrado))
	for q in (quadrado.reverter,quadrado.negativar):
		m = len(p)
		while m > 0:
			m -= 1
			p.append(q(p[m]))
	while len(p) < n:
		p.append(quadrado.gerar(quadrado.Quadrado))
	return p

def sortear (p,n=range(2)):
	p.sort()
	p,t = p.populacao,p.f
	c = len(p)
	#print(p.f)
	while c > 0:
		r = []
		f = t
		for d in n:
			d = 0
			a = quadrado.tela.random() * f
			while d < len(p):
				if p[d].f != None and not d in r:
					if a < p[d].f:
						break
					a -= p[d].f
				d += 1
			try:
				f -= p[d].f
				r.append(d)
			except IndexError:
				print('ERRO',len(p))
		yield r
		c -= 1

class Quadro:

	populacao = correr = fila = f = s = None

	def __contains__ (self,algo):
		return self.populacao.__contains__(algo)
	def __eq__ (self,outro):
		try:
			return self.populacao.__eq__(outro.populacao) #and self.s == outro.s
		except Exception:
			return False
	def __ne__ (self,outro):
		return self.__lt__(outro) or self.__gt__(outro) #self.populacao.__ne__(outro.populacao)
	def __le__ (self,o):
		try:
			return self.__eq__(o) or self.populacao.__lt__(o.populacao)
		except AttributeError:
			return NotImplemented
	def __ge__ (self,o):
		try:
			return self.__eq__(o) or self.populacao.__gt__(o.populacao)
		except AttributeError:
			return NotImplemented
	def __gt__ (self,o):
		return not self.__le__(o)
	def __lt__ (self,o):
		return not self.__ge__(o)
	def __add__ (self,o):
		return Quadro(self.s,self.populacao+list(o))#.populacao
	def __iadd__ (self,o):
		self.populacao += list(o)#.populacao
		return self
	def __imul__ (self,o):
		self.populacao *= o
		return self
	def __mul__ (self,o):
		return Quadro(self.s,self.populacao*o)
	def __len__ (self):	
		return self.populacao.__len__()
	def __repr__ (self):
		return self.__str__()
	def __str__ (self):
		res = 'Quadro(populacao=%s)' %self.populacao.__repr__()
		if __name__ != '__main__':
			res = 'quadro.%s' %res
		return res
	def __iter__ (self):
		return self.populacao.__iter__()
	
	def __init__ (self, mestra=None, populacao=None):#16):#2):
		if type(mestra) in (Quadro,quadrado.Quadrado):
			if populacao == None:
				try:
					populacao = mestra.populacao
				except AttributeError:
					if mestra.s != None and mestra.s.n != None:
						populacao = mestra.s.n.populacao
			mestra = mestra.s
		if not type(populacao) in (list,tuple,set):
			if populacao == None:
				populacao = tamanho
			populacao = popular(populacao)
		elif type(populacao) != list:
			populacao = list(populacao)
		self.populacao = populacao
	#	self.pedidos = []
		self.mestra(mestra)
		if mestra == None:
			mestra = quadrado.tela.Tela(None,self)

	def mestra (self, mestra=None):
		if None == mestra:
			mestra = self.s
			if None == self.s:
				return
		else:
			self.s = mestra
			if None == mestra.q:
				self.s.protagonista(quadrado.Quadrado(mestra=mestra))

		if type(self.populacao) in (set,tuple,list):
			if type(self.populacao) != list:
				self.populacao = list(self.populacao)
		c = len(self.populacao)
		while c > 0:
			c += -1
			if type(self.populacao[c]) != quadrado.Quadrado:
				self.populacao[c] = quadrado.Quadrado(self.populacao[c])
			self.populacao[c].mestra(mestra,quadrado.tela.PRE)

	def continuar (self,quem=None):
		self.pausar()
		#print(quem)
		quadrado.tela.sleep(1/11)#1.44)
	#	if len(self.pedidos) > 0:
		if self.fila != None:
			return
		self.fila = quem
		t = []
		try:
			for x,y in sortear(self):
				#print(x,y)
				t.append(quadrado.cruzar(self.populacao[x],self.populacao[y]))
		except ValueError:
			t.extend(self.populacao)	
		t.append(self.populacao[0].copy())
		t,self.populacao = self.populacao,t
		self.mestra()
		self.s.q.ir(para=(quadrado.tela.qx,quadrado.tela.qy),permissao=True)
		self.s.continuar(t)
	#	self.iniciar()
	#	self.pedidos.clear()
		self.fila = None
		print(len(self),'no plano.')
	
	
	def reiniciar (self,event=False):
		self.pausar()
		self.populacao.clear()
		self.__init__(self.s)
		print(len(self),'novos quadrados gerados')
		if event:
			self.iniciar()
	
	def iniciar (self):
		if self.s.q in self:# and self.populacao[self.populacao.index(self.s.q)].find('100001000') == 0:
		#	a = self.populacao[self.index(self.s.q)]
			#print(a, a.x==self.s.q.x,a.y==self.s.q.y)
			self.reiniciar()
		self.s.q.iniciar()
		self.correr = True
		try:
			for q in self:#.populacao:
				q.iniciar()
		except RuntimeError:
			self.s.reiniciar()

	def pausar (self):
		self.correr = False
		self.s.q.pausar()
		for q in self:#.populacao:
			q.pausar()

	def avaliar (self):
		self.f = 0
		for ind in self:#.populacao:
			self.f += ind.avaliar()
		return self.f

	def sort (self,decrescente=True,naoavalie=False):
		if not naoavalie:
			self.avaliar()
		self.populacao.sort()
		if decrescente:
			self.reverse()
	
	def reverse (self):
		self.populacao.reverse()

	def remove (self,o):
		return self.populacao.remove(o)
	def pop (self,i):
		return self.populacao.pop(i)
	def index (self, qual):
		return self.populacao.index(qual)
	def count (self, qual):
		return self.populacao.count(qual)
	def append (self,quad):
		return self.populacao.append(quad)
	def extend (self,quad):
		return self.populacao.extend(quad)
	def copy (self):
		return Quadro(self)



import setuptools
with open("README.md", "r", encoding="utf-8") as fl:
    long_description = fl.read()

setuptools.setup(
    name='Aboutn',
    version='0.2.2.7',
    url='https://github.com/AkiraDemenech/About-n-Squares/',
    license='MIT License',
    author='Guilherme Akira Demenech Mori',
    author_email='akira.demenech@gmail.com',
    keywords='art constructivism game',
    description="An Art and Genetic Algorithms Exploration Package based on Russian Constructivism bidimensional graphic works, ideas and method; ",
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=setuptools.find_packages(),
    install_requires=[],
    classifiers=[ "Programming Language :: Python :: 3", "License :: OSI Approved :: MIT License", "Operating System :: OS Independent", ],
)


from Aboutn import quadrado
from Aboutn import quadro
tela = quadrado.tela

def iniciar (classe=tela.Tela):
	try:
		classe().s.iniciar()
	except AttributeError:
		classe().iniciar()

#if __name__ == "__main__":
#    quadrado.Quadrado().s.iniciar()
#    quadro.Quadro().s.iniciar()
#    tela.Tela().iniciar()